function broadcast(componentName, eventName, params) {
  this.$children.forEach((child) => {
    var name = child.$options.componentName;
    if (name === componentName) {
      child.$emit.apply(child, [eventName].concat(params));
    } else {
      broadcast.apply(child, [componentName, eventName].concat([params]));
    }
  });
}
const emitter = {
  methods: {
    dispatch(componentName, eventName, params) {
      var parent = this.$parent || this.$root;
      var name = parent.$options.componentName;
      while (parent && (!name || name !== componentName)) {
        parent = parent.$parent;
        if (parent) {
          name = parent.$options.componentName;
        }
      }
      if (parent) {
        parent.$emit.apply(parent, [eventName].concat(params));
      }
    },
    broadcast(componentName, eventName, params) {
      broadcast.call(this, componentName, eventName, params);
    }
  }
};
function normalizeComponent(scriptExports, render3, staticRenderFns, functionalTemplate, injectStyles, scopeId, moduleIdentifier, shadowMode) {
  var options = typeof scriptExports === "function" ? scriptExports.options : scriptExports;
  if (render3) {
    options.render = render3;
    options.staticRenderFns = staticRenderFns;
    options._compiled = true;
  }
  if (scopeId) {
    options._scopeId = "data-v-" + scopeId;
  }
  return {
    exports: scriptExports,
    options
  };
}
const _sfc_main$1 = {
  name: "IFunLazySelect",
  mixins: [emitter],
  inheritAttrs: false,
  props: {
    // 初始选中的值
    checked: {
      type: [String, Array],
      default: ""
    },
    // 所有的数据源
    data: {
      type: Array,
      required: true,
      default: () => []
    },
    // 下拉渲染options的属性
    selectProps: {
      type: Object,
      default: () => ({
        value: "value",
        label: "label"
      })
    },
    // 是否进行远程加载
    lazy: {
      type: Boolean,
      default: false
    },
    // 分页查询时，调用外部函数
    load: {
      type: Function
    }
  },
  data() {
    const $this = this;
    return {
      // 用户下拉选择
      page: 1,
      size: 15,
      lazyOption: {
        loadData: $this.loadData,
        distance: 20,
        scrollBody: ".el-scrollbar__wrap",
        // 为el-select 滚动容器的DOM元素的class选择器
        callback: (fn) => {
          $this.$once("hook:beforeDestroy", () => fn());
        }
      },
      // 默认展示数据源，保留props数据源
      filterData: []
    };
  },
  watch: {
    data: {
      deep: true,
      immediate: true,
      handler(data) {
        this.filterData = [...data];
      }
    }
  },
  computed: {
    isMulti() {
      return this.$attrs.multiple;
    },
    /**
     * 当前下拉展示的
     */
    viewData() {
      if (this.lazy) {
        return this.filterData;
      }
      const data = this.filterData.slice(0, this.size * this.page);
      if (this.checked === "" || this.checked === void 0 || this.checked === null || this.checked.length < 1) {
        return data;
      }
      const markValue = [];
      let info = data.reduce((arr, item) => {
        const value = item[this.selectProps.value];
        let bool = false;
        if (this.isMulti) {
          bool = this.checked.includes(value);
        } else {
          bool = this.checked === value;
        }
        if (bool) {
          markValue.push(value);
          arr.push(item);
        }
        return arr;
      }, []);
      if (info && (this.isMulti ? markValue.length === this.checked : markValue.length === 1)) {
        return data;
      }
      info = this.data.reduce((arr, item) => {
        const value = item[this.selectProps.value];
        if (markValue.includes(value)) {
          return arr;
        }
        let bool = false;
        if (this.isMulti) {
          bool = this.checked.includes(value);
        } else {
          bool = this.checked === value;
        }
        if (bool) {
          arr.push(item);
        }
        return arr;
      }, []);
      if (info) {
        data.unshift(...info);
      }
      return data;
    }
  },
  mounted() {
  },
  methods: {
    handleOptionVisible(bool) {
      if (!bool) {
        this.handleFilterData("");
      }
      this.$emit("visible-change", bool);
    },
    handleFilterData(str) {
      if (str === "") {
        this.filterData = this.data;
        return;
      }
      this.filterData = this.data.filter(
        (item) => item[this.selectProps.label].includes(str)
      );
    },
    loadData() {
      if (this.lazy) {
        this.load();
        return;
      }
      if (this.size * this.page >= this.data.length) {
        return;
      }
      this.page++;
    }
  }
};
var _sfc_render$1 = function render() {
  var _vm = this, _c = _vm._self._c;
  return _c("div", { staticClass: "ifun-lazy-select" }, [_c("el-select", _vm._g(_vm._b({ directives: [{ name: "lazy-load", rawName: "v-lazy-load", value: _vm.lazyOption, expression: "lazyOption" }], staticStyle: { "width": "100%" }, attrs: { "filter-method": _vm.handleFilterData }, on: { "visible-change": _vm.handleOptionVisible } }, "el-select", _vm.$attrs, false), _vm.$listeners), _vm._l(_vm.viewData, function(item) {
    return _c("el-option", { key: item[_vm.selectProps.value], attrs: { "value": item[_vm.selectProps.value], "label": item[_vm.selectProps.label] } });
  }), 1)], 1);
};
var _sfc_staticRenderFns$1 = [];
var __component__$1 = /* @__PURE__ */ normalizeComponent(
  _sfc_main$1,
  _sfc_render$1,
  _sfc_staticRenderFns$1,
  false,
  null,
  "74443525"
);
const LazySelect = __component__$1.exports;
function throttle(fn, delay) {
  let timer = null;
  return () => {
    if (timer) {
      return;
    }
    timer = setTimeout(() => {
      fn(...arguments);
      timer = null;
    }, delay);
  };
}
const VLazyLoad = {
  /**
   * el - 指令所绑定的元素DOM
   * binding - 传入指令的其他附带参数
   *    name - 指令名
   *    value  - 指令绑定的值
   *    oldValue - 绑定的前一个值
   *    expression - 指令绑定的字符串形式的表达式
   *    arg - 传给指令的参数
   *    modifiers - 指令修饰符的对象
   * vnode
   * oldVnode
   */
  inserted: (el, binding) => {
    const { loadData, distance, scrollBody, callback } = binding.value;
    let scrollContainer = el;
    if (scrollBody) {
      scrollContainer = el.querySelector(scrollBody) || el;
    }
    const scroll = throttle(
      handleScroll.bind(null, scrollContainer, { distance, loadData }),
      500
    );
    scrollContainer.addEventListener("scroll", scroll);
    const fn = () => {
      scrollContainer.removeEventListener("scroll", scroll);
    };
    callback(fn);
  }
};
function handleScroll(dom, option) {
  const scrollBottom = dom.scrollTop + dom.clientHeight;
  if (dom.scrollHeight - scrollBottom <= option.distance) {
    option.loadData();
  }
}
const _sfc_main = {
  name: "IFunLazyList",
  inheritAttrs: false,
  props: {
    // 所有的数据源
    data: {
      type: Array,
      required: true,
      default: () => []
    },
    size: {
      type: Number,
      default: () => 15
    },
    // 渲染的属性
    options: {
      type: Object,
      default: () => ({
        value: "value",
        label: "label"
      })
    }
  },
  data() {
    const $this = this;
    return {
      // 用户下拉选择
      page: 1,
      lazyOption: {
        loadData: $this.loadData,
        distance: 20,
        callback: (fn) => {
          $this.$once("hook:beforeDestroy", () => fn());
        }
      }
    };
  },
  computed: {
    /**
     * 当前下拉展示的
     */
    viewData() {
      const data = this.data.slice(0, this.size * this.page);
      return data;
    }
  },
  mounted() {
  },
  methods: {
    loadData() {
      if (this.size * this.page >= this.data.length) {
        return;
      }
      this.page++;
    }
  }
};
var _sfc_render = function render2() {
  var _vm = this, _c = _vm._self._c;
  return _c("div", { directives: [{ name: "lazy-load", rawName: "v-lazy-load", value: _vm.lazyOption, expression: "lazyOption" }], staticClass: "ifun-lazy-list" }, [_c("ul", { ref: "box", staticClass: "list-box" }, _vm._l(_vm.viewData, function(item, index) {
    return _c("li", { key: item[_vm.options.value] || index, staticClass: "list-item" }, [_vm.$attrs.params ? _vm._t("default", null, { "params": item }) : _c("span", [_vm._v(_vm._s(item[_vm.options.label] || item))])], 2);
  }), 0)]);
};
var _sfc_staticRenderFns = [];
var __component__ = /* @__PURE__ */ normalizeComponent(
  _sfc_main,
  _sfc_render,
  _sfc_staticRenderFns,
  false,
  null,
  "7413027b"
);
const LazyList = __component__.exports;
const entry = {
  install(app) {
    app.component(LazySelect.name, LazySelect);
    app.component(LazyList.name, LazyList);
    app.directive("lazy-load", VLazyLoad);
  }
};
export {
  entry as default
};
//# sourceMappingURL=lazy-load.esm.js.map
